////////////////////////////////////////////////////////////////////////////////
// A script for checking correctness of models that are created by using 
// a UML profile for developing databases that conform to The Third Manifesto.
// 
////////////////////////////////////////////////////////////////////////////////

//Function trim is from the CodeGuru Forums and helps us to remove spaces from the beginning and the end of a string.
//Link: http://www.codeguru.com/forum/archive/index.php/t-204545.html
//Author: anupam kant
//Posted: August 14th, 2002, 05:26 AM
function trim(str)
{
str = str.toString();
while (1)
{
if (str.substring(0, 1) != " ")
{
break;
}
str = str.substring(1, str.length);
}
while (1)
{
if (str.substring(str.length - 1,str.length) != " ")
{
break;
}
str = str.substring(0, str.length - 1);
}
return str;
}

function Split(list){
   var ss = list.split(",");
   return(ss);
}

//Program that checks whether The Third Manifesto (TTM) database design models correspond to the underlying constraints of TTM.
var app, prj;
var is_component;
var is_candidate_key;
var is_candidate_key_blank;
var is_referenced_key_blank;
var is_referencing_key_blank;
var amount_of_keys;
var atr1, atr2;

app = new ActiveXObject("StarUML.StarUMLApplication");
prj = app.GetProject();

app.AddMessageItem(1, "Checking constraints...", app.GetProject());
VisitOwnedElement(prj);

//Function that visits all the elements

function VisitOwnedElement(owner){
    var elem;
    for (var i = 0; i < owner.GetOwnedElementCount(); i++){
        elem = owner.GetOwnedElementAt(i);
	
        if (elem.IsKindOf("UMLClass")){
    	    var stereotypeStr = elem.StereotypeName;

	//Classes with no stereotype
	if (stereotypeStr == ""){
		app.AddMessageItem(0, "Error in '"+elem.Name+ "': diagrams cannot contain classes that have no stereotype", app.GetProject());
	}
	
	//Names of attributes in Tuple type, Relvar type, Base relvar, Virtual relvar, and Snapshot must be unique within a given class.
	if (stereotypeStr == "Relation type" || stereotypeStr == "Tuple type" || stereotypeStr == "Base relvar" || stereotypeStr == "Snapshot" || stereotypeStr == "Virtual relvar"){
		var attributes = new Array();
		var colCount = elem.MOF_GetCollectionCount("Attributes");
		var amount_of_attributes = colCount;
		for (var j = 0; j < colCount; j++){
			var colItem = elem.MOF_GetCollectionItem("Attributes", j);
			attributes[j] = colItem.Name;
		}
		attributes.sort();
		for (var k = 0; k<attributes.length-1; k++) {
			if (attributes[k]==attributes[k+1]) {
				app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Attribute '"+attributes[k]+"'): attributes must have unique names within a class", app.GetProject());
				break;
			}
		}
	}
	
	//Names of components must be unique within a class that is used to represent a Scalar type
	if (stereotypeStr == "User-defined scalar type"){
		var attributes = new Array();
		var colCount = elem.MOF_GetCollectionCount("Attributes");
		for (var j = 0; j < colCount; j++){
			var colItem = elem.MOF_GetCollectionItem("Attributes", j);
			var stereotypeStr2 = colItem.StereotypeName;
			if (stereotypeStr2 == "Component" ) {
				attributes[j] = colItem.Name;
			}
		}
		attributes.sort();
		for (var k = 0; k<attributes.length-1; k++) {
			if (attributes[k]==attributes[k+1]) {
				app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Component '"+attributes[k]+"'): components must have unique names within a class", app.GetProject());
				break;
			}
		}
	}
	
//User-defined scalar type

	    var tagCount = 0;
	    var tagCountCheck = 0;
		
    	    if (stereotypeStr == "User-defined scalar type"){
		is_component =0;
		var colCount = elem.MOF_GetCollectionCount("Attributes");
			for (var l = 0; l < colCount; l++){
				var colItem = elem.MOF_GetCollectionItem("Attributes", l);
				var stereotypeStr2 = colItem.StereotypeName;
				if (stereotypeStr2 == "Component" && is_component==0) {
					is_component=1;
				}
				if (stereotypeStr2 != "Component"){ 
					app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Attribute '"+colItem.Name+"'): user-defined scalar types can have only attributes with stereotype: Component", app.GetProject());
				}
				//Multiplicity of components must be 1
				var multi = colItem.MOF_GetAttribute("Multiplicity");
				if (stereotypeStr2 == "Component" && multi != "1"){
					app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Component '"+colItem.Name+"'): components of a possible representation can only have multiplicity: 1", app.GetProject());
				}
				
				//Components of a user-defined scalar type cannot have an initial value
				var init = colItem.MOF_GetAttribute("InitialValue");		    
				if (stereotypeStr2 == "Component" && init != ""){
					app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Component '"+colItem.Name+"'): components of a user-defined scalar type cannot have an initial value", app.GetProject());
				}
				
				var type = colItem.TypeExpression;
				//Recursively defined scalar types are not permitted
				if (stereotypeStr2 == "Component" && type == elem.Name){
					app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Component '"+colItem.Name+"'): recursively defined scalar types are not permitted. A component of a scalar type type cannot have the scalar type as its type", app.GetProject());
				}
				//Each component must have a type
				if (stereotypeStr2 == "Component" && type == ""){
					app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Component '"+colItem.Name+"'): each component must have a type", app.GetProject());
				}
				
			}
			//User-defined scalar type must have at least one component
			if (is_component==0) { 
				app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "': each user-defined scalar type must have at least one Component", app.GetProject());
			}
			
			//User-defined scalar types can have only operations with stereotype Type constraint
			colCount = elem.MOF_GetCollectionCount("Operations");
			for (var m = 0; m < colCount; m++){
				colItem = elem.MOF_GetCollectionItem("Operations", m);
				stereotypeStr2 = colItem.StereotypeName;
				if (stereotypeStr2 != "Type constraint"){
					app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Operation '"+colItem.Name+"'): user-defined scalar types can have only operations with stereotype: Type constraint", app.GetProject());
				}
			}    	    
	    }

//Tuple type

    	    if (stereotypeStr == "Tuple type"){
		colCount = elem.MOF_GetCollectionCount("Attributes");
		for (var n = 0; n < colCount; n++){
    		    colItem = elem.MOF_GetCollectionItem("Attributes", n);
		    var stereotypeStr2 = colItem.StereotypeName;
		     //Only scalar types have components
		    if (stereotypeStr2 == "Component"){ 
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Attribute '"+colItem.Name+"'): only scalar types have components of a possible representation", app.GetProject());
		    }
		    //Multiplicity of attributes must be 1
    		    var multi = colItem.MOF_GetAttribute("Multiplicity");		    
		    if (multi != "1"){
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Attribute '"+colItem.Name+"'): attributes of a tuple type can only have multiplicity: 1", app.GetProject());
		    }
		    
		    var type = colItem.TypeExpression;
		    //Recursively defined headings are not permitted
		    if (type == elem.Name){
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Attribute '"+colItem.Name+"'): recursively defined headings are not permitted. An attribute of a tuple type cannot have the tuple type as its type", app.GetProject());
		    }
		    //Each attribute must have a type
		    if (type == ""){
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Attribute '"+colItem.Name+"'): each attribute must have a type", app.GetProject());
		    }
		    
		    //Attributes of a tuple type cannot have an initial value
    		    var init = colItem.MOF_GetAttribute("InitialValue");		    
		    if (init != ""){
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Attribute '"+colItem.Name+"'): attributes of a tuple type cannot have an initial value", app.GetProject());
		    }
		} 
		//Tuple types cannot have operations
		colCount = elem.MOF_GetCollectionCount("Operations");
		if (colCount > 0){
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "': tuple types cannot have operations", app.GetProject());
		}		  			
	    }
	    
//Relation type

    	    if (stereotypeStr == "Relation type"){
		colCount = elem.MOF_GetCollectionCount("Attributes");
		for (var o = 0; o < colCount; o++){
    		    colItem = elem.MOF_GetCollectionItem("Attributes", o);
		    var stereotypeStr2 = colItem.StereotypeName;
		    //Only scalar types have components
		    if (stereotypeStr2 == "Component"){ 
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Attribute '"+colItem.Name+"'): only scalar types have components of a possible representation", app.GetProject());
		    }
		    //Multiplicity of attributes must be 1
    		    var multi = colItem.MOF_GetAttribute("Multiplicity");		    
		    if (multi != "1"){
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Attribute '"+colItem.Name+"'): attributes of a relation type can only have multiplicity: 1", app.GetProject());
		    }
		    
		    var type = colItem.TypeExpression;
		    //Recursively defined headings are not permitted
		    if (type == elem.Name){
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Attribute '"+colItem.Name+"'): recursively defined headings are not permitted. An attribute of a relation type cannot have the relation type as its type", app.GetProject());
		    }
		    //Each attribute must have a type
		    if (type == ""){
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Attribute '"+colItem.Name+"'): each attribute must have a type", app.GetProject());
		    }
		    
		    //Attributes of a relation type cannot have an initial value
    		    var init = colItem.MOF_GetAttribute("InitialValue");		    
		    if (init != ""){
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Attribute '"+colItem.Name+"'): attributes of a relation type cannot have an initial value", app.GetProject());
		    }
		} 
		//Relation types cannot have operations
		colCount = elem.MOF_GetCollectionCount("Operations");
		if (colCount > 0){
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "': relation types cannot have operations", app.GetProject());
		}		  			
	    }

//Base relvar, snapshot, and virtual relvar

    	    if (stereotypeStr == "Base relvar" || stereotypeStr == "Virtual relvar" || stereotypeStr == "Snapshot"){
		colCount = elem.MOF_GetCollectionCount("Operations");
		is_candidate_key =0;
		amount_of_keys=0;
		for (var p = 0; p < colCount; p++){
    		    colItem = elem.MOF_GetCollectionItem("Operations", p);
		    var stereotypeStr3 = colItem.StereotypeName;
			if (stereotypeStr3 == "Candidate key") {
				amount_of_keys=amount_of_keys+1;
			}
			if (stereotypeStr3 == "Candidate key" && is_candidate_key==0) {
				is_candidate_key=1;
			}
		    //Relvars can only have operations that denote Candidate keys or Foreign keys
		    if (stereotypeStr3 != "Candidate key" && stereotypeStr3 != "Foreign key"){
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Operation '"+colItem.Name+"'): relvars can have only operations with stereotypes: Candidate key, Foreign key", app.GetProject());
		    }
		}
		//Each relvar must have at least one Candidate key
		if (is_candidate_key==0){ 
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "': each relvar must have at least one Candidate key", app.GetProject());
		}
		
		//Relvar with no attributes has eactly one key and it is an empty key
		if (amount_of_keys>1 && amount_of_attributes==0){ 
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "': relvar with no attributes must have exactly one Candidate key and it is an empty key", app.GetProject());
		}
		
		colCount = elem.MOF_GetCollectionCount("Attributes");
		for (var r = 0; r < colCount; r++){
    		    colItem = elem.MOF_GetCollectionItem("Attributes", r);
		    //Only scalar types have components
		    var stereotypeStr2 = colItem.StereotypeName;
		    if (stereotypeStr2 == "Component"){ 
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Attribute '"+colItem.Name+"'): only scalar types have components of a possible representation", app.GetProject());
		    }
		    //Multiplicity of attributes must be 1
    		    var multi = colItem.MOF_GetAttribute("Multiplicity");
		    if (multi != "1"){
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Attribute '"+colItem.Name+"'): attributes of a relvar can only have multiplicity: 1", app.GetProject());
		    }
		    
		    var type = colItem.TypeExpression;
		    //Each attribute must have a type
		    if (type == ""){
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Attribute '"+colItem.Name+"'): each attribute must have a type", app.GetProject());
		    }
		}
		//tagged values
		tagCountCheck = 0;
		tagCount = elem.GetTaggedValueCount();
		if (stereotypeStr == "Virtual relvar") {
		    for (var a = 0; a < tagCount; a++) {
			var tagItem = elem.GetTaggedValueAt(a);
			if (tagItem.Name == "relational_expression") {
			    tagCountCheck++;
			    if (trim(tagItem.DataValue) == "") {
				app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "': tagged values cannot be blank", app.GetProject());
			    }
			}
		    }
		    if (tagCountCheck < 1) {
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "': tagged values must be set", app.GetProject()); 
		    }
		} 
		if (stereotypeStr == "Snapshot"){
		    for (var b = 0; b < tagCount; b++) {
			var tagItem = elem.GetTaggedValueAt(b);
			if (tagItem.Name == "relational_expression") {
			    tagCountCheck++;
			    if (trim(tagItem.DataValue) == "") {
				app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "': tagged values cannot be blank", app.GetProject());
			    }
			}
			if (tagItem.Name == "now_and_then") {
			    tagCountCheck++;
			    if (trim(tagItem.DataValue) == "") {
				app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "': tagged values cannot be blank", app.GetProject());
			    }
			}
		    }
		    if (tagCountCheck < 2) {
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "': tagged values must be set", app.GetProject()); 
		    }
		}  		
	    }

//Sets of operators

    	    if (stereotypeStr == "Set of operators"){
		colCount = elem.MOF_GetCollectionCount("Attributes");
		//Sets of operators cannot have attributes
		if (colCount > 0){
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "': sets of operators cannot have attributes", app.GetProject());
		}
		//Set of operators can have only operations User-defined read-only operator or User-defined update operator
		colCount = elem.MOF_GetCollectionCount("Operations");
		for (var s = 0; s < colCount; s++){
    		    colItem = elem.MOF_GetCollectionItem("Operations", s);
		    var stereotypeStr3 = colItem.StereotypeName;
		    if (stereotypeStr3 != "User-defined read-only operator" && stereotypeStr3 != "User-defined update operator"){
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '" +elem.Name+ "' (Operation '"+colItem.Name+"'): sets of operators can only have operations with stereotypes: User-defined read-only operator, User-defined update operator", app.GetProject());
		    }
		}	    
	    }


//Sets of constraints

    	    if (stereotypeStr == "Set of constraints"){
		colCount = elem.MOF_GetCollectionCount("Attributes");
		//Sets of constraints cannot have attributes
		if (colCount > 0){
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '" +elem.Name+ "': sets of constraints cannot have attributes", app.GetProject());
		} 
		//Set of constraints can only have constraints Relvar constraint or Database constraint
		colCount = elem.MOF_GetCollectionCount("Operations");
		for (var t = 0; t < colCount; t++){
    		    colItem = elem.MOF_GetCollectionItem("Operations", t);
		    var stereotypeStr3 = colItem.StereotypeName;
		    if (stereotypeStr3 != "Relvar constraint" && stereotypeStr3 != "Database constraint"){
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '" +elem.Name+ "' (Operation '"+colItem.Name+"'): sets of constraints can only have operations with stereotypes: Relvar constraint, Database constraint", app.GetProject());
		    }
		}	    
	    }

//Constraints

	    var colCount2 = elem.MOF_GetCollectionCount("Operations");
	    for (var u = 0; u < colCount2; u++){
    		colItem = elem.MOF_GetCollectionItem("Operations", u);
		var stereotypeStr4 = colItem.StereotypeName;
		//Constraints cannot have parameters
		if (stereotypeStr4 == "Type constraint" || stereotypeStr4 == "Relvar constraint" || stereotypeStr4 == "Database constraint" || stereotypeStr4 == "Candidate key" || stereotypeStr4 == "Foreign key") {
		    colCount = colItem.MOF_GetCollectionCount("Parameters");
		    if (colCount > 0){
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Constraint '"+colItem.Name+"'): constraints cannot have parameters", app.GetProject());
		    }
		    //tagged values
		    is_candidate_key_blank=0;
		    tagCountCheck = 0;
		    tagCount = colItem.GetTaggedValueCount();
		    if (stereotypeStr4 == "Candidate key") {
			for (var c = 0; c < tagCount; c++) {
			    var tagItem = colItem.GetTaggedValueAt(c);
			    if (tagItem.Name == "attributes_in_key") {
				tagCountCheck++;
				if (trim(tagItem.DataValue) == "") {
					is_candidate_key_blank++;
				}
			    }
		        }
			
			//If relvar has one or more attributes and have an empty key, then it can be the only key of the relvar
			if ((is_candidate_key_blank==1 || tagCountCheck < 1)  && amount_of_attributes>0 && amount_of_keys>1) {
				app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (" + stereotypeStr4 + " '"+colItem.Name+ "'): if relvar has an empty key, then it must be the only key of the relvar", app.GetProject());
			}
			
		    } 
		    if (stereotypeStr4 == "Relvar constraint" || stereotypeStr4 == "Database constraint") {
			for (var d = 0; d < tagCount; d++) {
			    var tagItem = colItem.GetTaggedValueAt(d);
			    if (tagItem.Name == "boolean_expression") {
				tagCountCheck++;
				if (trim(tagItem.DataValue) == "") {
				    app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (" + stereotypeStr4 + " '"+colItem.Name+ "'): tagged values cannot be blank", app.GetProject());
				}
			    }
		        }
			if (tagCountCheck < 1) {
			    app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (" + stereotypeStr4 + " '"+colItem.Name+ "'): tagged values must be set", app.GetProject()); 
		        }
		    } 
		    if (stereotypeStr4 == "Type constraint") {
			for (var e = 0; e < tagCount; e++) {
			    var tagItem = colItem.GetTaggedValueAt(e);
			    if (tagItem.Name == "boolean_expression") {
				tagCountCheck++;
				if (trim(tagItem.DataValue) == "") {
				    app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (" + stereotypeStr4 + " '"+colItem.Name+ "'): tagged values cannot be blank", app.GetProject());
				}
			    }
			    if (tagItem.Name == "name_of_possrep") {
				tagCountCheck++;
			        if (trim(tagItem.DataValue) == "") {
				    app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (" + stereotypeStr4 + " '"+colItem.Name+ "'): tagged values cannot be blank", app.GetProject());
				}
			    }
			} 
			if (tagCountCheck < 2) {
			    app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (" + stereotypeStr4 + " '"+colItem.Name+ "'): tagged values must be set", app.GetProject());
		        }
		    }
		    if (stereotypeStr4 == "Foreign key") {
			tagCountCheck=0;
			var attributesInKey, referencedRelvarKey;			
			var is_referencing_key_not_missing=0;
			var is_referenced_relvar_not_missing=0;
			var is_referenced_key_not_missing=0;
			for (var f = 0; f < tagCount; f++) {
			    var tagItem = colItem.GetTaggedValueAt(f);
			    if (tagItem.Name == "attributes_in_key") {
				is_referencing_key_not_missing++;
				if (trim(tagItem.DataValue) == "") {
					is_referencing_key_not_missing--;
				}
				attributesInKey = tagItem.DataValue;
			    }
			    if (tagItem.Name == "referenced_relvar") {
				is_referenced_relvar_not_missing++;
				if (trim(tagItem.DataValue) == "") {
					is_referenced_relvar_not_missing--;				    
				}
			    }
			    if (tagItem.Name == "attributes_in_referenced_relvar_key") {				
				is_referenced_key_not_missing++;
				if (trim(tagItem.DataValue) == "") {
					is_referenced_key_not_missing--;
				}
				referencedRelvarKey = tagItem.DataValue;
			    }			    
		        }
						
			if (is_referencing_key_not_missing!=0 && is_referenced_key_not_missing==0) {
				app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (" + stereotypeStr4 + " '"+colItem.Name+ "'): an empty candidate key can be referred only by an empty foreign key", app.GetProject());
			}			
			if (is_referencing_key_not_missing==0 && is_referenced_key_not_missing!=0) {
				app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (" + stereotypeStr4 + " '"+colItem.Name+ "'): an empty foreign key can refer only to an empty candidate key", app.GetProject());
			}			
			if (is_referenced_relvar_not_missing==0) {
				app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (" + stereotypeStr4 + " '"+colItem.Name+ "'): tagged value that represents referenced relvar must be set", app.GetProject());
			}			
			var attributesInKey_array, referencedRelvarKey_array, attributesInKey_amt=0, referencedRelvarKey_amt=0;
			if (is_referencing_key_not_missing!=0) {
				attributesInKey_array = Split(attributesInKey);
			}
			if (is_referenced_key_not_missing!=0) {
				referencedRelvarKey_array = Split(referencedRelvarKey);
			}
			for (var key in attributesInKey_array) {
				attributesInKey_amt++;
			}
			for (var key in referencedRelvarKey_array) {
				referencedRelvarKey_amt++;
			}
			if (attributesInKey_amt!=referencedRelvarKey_amt) {
				app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (" + stereotypeStr4 + " '"+colItem.Name+ "'): tagged values attributes_in_key and attributes_in_referenced_relvar_key must refer to the same amount of attributes", app.GetProject()); 
			}				
		    }
	        }

	    
//DirectionKinds: IN = 0, INOUT = 1, OUT = 2, RETURN = 3
//User-defined read-only operators

		if (stereotypeStr4 == "User-defined read-only operator"){
		    //Each specification of an operator must include specification of its body
		    var spec = colItem.MOF_GetAttribute("Specification");
		    if (trim(spec) == "") {
				app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (" + stereotypeStr4 + " '"+colItem.Name+ "'): specification of the body cannot be blank (property Specification of the operation)", app.GetProject());
		    }
		    var dir_ret = 0;
		    var parameters = new Array();
		    colCount = colItem.MOF_GetCollectionCount("Parameters");
		    for (var v = 0; v < colCount; v++){		    
			var parItem = colItem.MOF_GetCollectionItem("Parameters", v);
			var direction = parItem.DirectionKind;
			if (direction == 3) {
			    dir_ret++;
			}
			//Parameters of a user-defined read-only operator must have DirectionKind IN or RETURN
			else if ((direction == 1) || (direction == 2)) {
			    app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Operator '"+colItem.Name+"', Parameter '"+parItem.Name+"'): parameters of a user-defined read-only operator must have DirectionKind IN or RETURN", app.GetProject());
			}
			
			var type = parItem.TypeExpression;
			//Each parameter must have a type
			if (type == ""){
				app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Operator '"+colItem.Name+"', Parameter '"+parItem.Name+"'): each parameter must have a type", app.GetProject());
			}
			parameters[v] = parItem.Name;
		    }
		    //Each user-defined read-only operator must have exactly one parameter with DirectionKind RETURN
		    if (dir_ret != 1) {
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Operator '"+colItem.Name+"', Parameter '"+parItem.Name+"'): each user-defined read-only operator must have exactly one parameter with DirectionKind RETURN", app.GetProject());
		    }
			parameters.sort();
			for (var kk = 0; kk<parameters.length-1; kk++) {
				if (parameters[kk]==parameters[kk+1]) {
					app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Parameter '"+parameters[kk]+"'): parameters must have unique names within an operation", app.GetProject());
					break;
				}
			}
		}

//User-defined update operators

		else if (stereotypeStr4 == "User-defined update operator"){
		    //Each specification of an operator must include specification of its body
		    var spec = colItem.MOF_GetAttribute("Specification");
		    if (trim(spec) == "") {
				app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (" + stereotypeStr4 + " '"+colItem.Name+ "'): specification of the body cannot be blank (property Specification of the operation)", app.GetProject());
		    }
		    var dir_in = 0;
		    var parameters = new Array();
		    colCount = colItem.MOF_GetCollectionCount("Parameters");
		    for (var w = 0; w < colCount; w++){		    
			var parItem = colItem.MOF_GetCollectionItem("Parameters", w);
			var direction = parItem.DirectionKind;
			if (direction == 0) {
			    dir_in++;
			}
			//All parameters of a user-defined update operator must have DirectionKind IN
			if (direction != 0) {
			    app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Operator '"+colItem.Name+"', Parameter '"+parItem.Name+"'): all parameters of a user-defined update operator must have DirectionKind IN", app.GetProject());
			}
			
			var type = parItem.TypeExpression;
			//Each parameter must have a type
			if (type == ""){
				app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Operator '"+colItem.Name+"', Parameter '"+parItem.Name+"'): each parameter must have a type", app.GetProject());
			}
			parameters[w] = parItem.Name;
		    }
		    //Each user-defined update operator must have at least one parameter with DirectionKind IN (at least one subject to update parameter)
		    if (dir_in == 0) {
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Operator '"+colItem.Name+"'): each user-defined update operator must have at least one parameter with DirectionKind IN (subject to update parameter)", app.GetProject());
		    }
			parameters.sort();
			for (var kkk = 0; kkk<parameters.length-1; kkk++) {
				if (parameters[kkk]==parameters[kkk+1]) {
					app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (Parameter '"+parameters[kk]+"'): parameters must have unique names within an operation", app.GetProject());
					break;
				}
			}
		    
		    
		    //tagged values
		    tagCountCheck = 0;
		    tagCount = colItem.GetTaggedValueCount();
		    for (var g = 0; g < tagCount; g++) {
			var tagItem = colItem.GetTaggedValueAt(g);
			if (tagItem.Name == "subject_to_update") {
			    tagCountCheck++;
			    if (trim(tagItem.DataValue) == "") {
				app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (" + stereotypeStr4 + " '"+colItem.Name+ "'): tagged values cannot be blank", app.GetProject());
			    }
			}
			}
			if (tagCountCheck < 1) {
			    app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (" + stereotypeStr4 + " '"+colItem.Name+ "'): tagged values must be set", app.GetProject()); 
			}
		}
	    }


//Attributes

	    var colCount3 = elem.MOF_GetCollectionCount("Attributes");
	    var possreps = new Array();
	    var amt_of_possreps=0;
	    for (var z = 0; z < colCount3; z++){
    		colItem = elem.MOF_GetCollectionItem("Attributes", z);
		var stereotypeStr5 = colItem.StereotypeName;
		if (stereotypeStr5 == "Component") {
		    tagCountCheck = 0;
		    tagCount = colItem.GetTaggedValueCount();
		    for (var h = 0; h < tagCount; h++) {
			var tagItem = colItem.GetTaggedValueAt(h);
			if (tagItem.Name == "name_of_possrep") {
			    possreps[amt_of_possreps]=tagItem.DataValue;
			    amt_of_possreps++;
			    tagCountCheck++;
			    if (trim(tagItem.DataValue) == "") {
				app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (" + stereotypeStr5 + " '"+colItem.Name+ "'): tagged values cannot be blank", app.GetProject());
			    }
			}
		    }
		    if (tagCountCheck < 1) {
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "' (" + stereotypeStr5 + " '"+colItem.Name+ "'): tagged values must be set", app.GetProject()); 
		    }
		}	    
	    }
	possreps.sort();
	var amt_of_possreps2=0;
	for (var c = 0; c<possreps.length; c++) {
		if (possreps[c]!=possreps[c+1]) {
			amt_of_possreps2++;
		}
	}
	//If a user-defined scalar type has more than one possible representations, then the class that represents the type must have tagged value init
	if (amt_of_possreps2>1) {
		tagCountCheck = 0;
		tagCount = elem.GetTaggedValueCount();
		    for (var a = 0; a < tagCount; a++) {
			var tagItem = elem.GetTaggedValueAt(a);
			if (tagItem.Name == "init") {
			    tagCountCheck++;
			    if (trim(tagItem.DataValue) == "") {
				app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "': if a scalar type has more than one possible representation, then tagged value init must be set", app.GetProject());
			    }
			}
		    }
		    if (tagCountCheck < 1) {
			app.AddMessageItem(0, "Error in " + stereotypeStr + " '"+elem.Name+ "': if a scalar type has more than one possible representation, then tagged value init must be set", app.GetProject()); 
		    }
	}
	}
	
	
//Recursion

	else if (elem.IsKindOf("UMLNamespace")) VisitOwnedElement(elem);      
    }
}
app.AddMessageItem(2, "Constraints checked!", app.GetProject());